@@ -649,6 +649,9 @@ class Redis(Client[AnyStr]):
         self._decodecontext: contextvars.ContextVar[
             Optional[bool],
         ] = contextvars.ContextVar("decode", default=None)
+        self._encodingcontext: contextvars.ContextVar[
+            Optional[str],
+        ] = contextvars.ContextVar("decode", default=None)
 
     @classmethod
     @overload
@@ -790,6 +793,7 @@ class Redis(Client[AnyStr]):
                 *args,
                 noreply=self.noreply,
                 decode=options.get("decode", self._decodecontext.get()),
+                encoding=self._encodingcontext.get(),
             )
             maybe_wait = await self._ensure_wait(command, connection)
             reply = await request
@@ -821,6 +825,7 @@ class Redis(Client[AnyStr]):
                 command,
                 *args,
                 decode=options.get("decode", self._decodecontext.get()),
+                encoding=self._encodingcontext.get(),
             )
             pool.release(_connection)
             reply = await request
@@ -838,19 +843,31 @@ class Redis(Client[AnyStr]):
                 pool.release(connection)
 
     @overload
-    def decoding(self, mode: Literal[False]) -> ContextManager[Redis[bytes]]:
+    def decoding(
+        self, mode: Literal[False], encoding: Optional[str] = None
+    ) -> ContextManager[Redis[bytes]]:
         ...
 
     @overload
-    def decoding(self, mode: Literal[True]) -> ContextManager[Redis[str]]:
+    def decoding(
+        self, mode: Literal[True], encoding: Optional[str] = None
+    ) -> ContextManager[Redis[str]]:
         ...
 
     @contextlib.contextmanager
-    def decoding(self, mode: bool) -> Iterator[Redis[Any]]:
+    @versionadded(version="4.8.0")
+    def decoding(
+        self, mode: bool, encoding: Optional[str] = None
+    ) -> Iterator[Redis[Any]]:
         """
         Context manager to temporarily change the decoding behavior
         of the client
 
+        :param mode: Whether to decode or not
+        :param encoding: Optional encoding to use if decoding. If not provided
+         the :paramref:`~coredis.Redis.encoding` parameter provided to the client will
+         be used.
+
         Example::
 
             client = coredis.Redis(decode_responses=True)
@@ -862,12 +879,15 @@ class Redis(Client[AnyStr]):
                     assert await client.get("fubar") == "baz"
 
         """
-        prev = self._decodecontext.get()
+        prev_decode = self._decodecontext.get()
+        prev_encoding = self._encodingcontext.get()
         self._decodecontext.set(mode)
+        self._encodingcontext.set(encoding)
         try:
             yield self
         finally:
-            self._decodecontext.set(prev)
+            self._decodecontext.set(prev_decode)
+            self._encodingcontext.set(prev_encoding)
 
     def monitor(self) -> Monitor[AnyStr]:
         """
@@ -902,10 +922,11 @@ class Redis(Client[AnyStr]):
     ) -> "coredis.pipeline.Pipeline[AnyStr]":
         """
         Returns a new pipeline object that can queue multiple commands for
-        later execution. :paramref:`transaction` indicates whether all commands
-        should be executed atomically. Apart from making a group of operations
-        atomic, pipelines are useful for reducing the back-and-forth overhead
-        between the client and server.
+        batch execution.
+
+        :param transaction: indicates whether all commands should be executed atomically.
+        :param watches: If :paramref:`transaction` is True these keys are watched for external
+         changes during the transaction.
         """
         from coredis.pipeline import Pipeline
 
@@ -915,17 +936,21 @@ class Redis(Client[AnyStr]):
         self,
         func: Callable[["coredis.pipeline.Pipeline[AnyStr]"], Coroutine[Any, Any, Any]],
         *watches: KeyT,
+        value_from_callable: bool = False,
+        watch_delay: Optional[float] = None,
         **kwargs: Any,
     ) -> Optional[Any]:
         """
         Convenience method for executing the callable :paramref:`func` as a
         transaction while watching all keys specified in :paramref:`watches`.
-        The :paramref:`func` callable should expect a single argument which is a
-        :class:`coredis.pipeline.Pipeline` object retrieved by calling
-        :meth:`~coredis.Redis.pipeline`.
+
+        :param func: callable should expect a single argument which is a
+         :class:`coredis.pipeline.Pipeline` object retrieved by calling
+         :meth:`~coredis.Redis.pipeline`.
+        :param watches: The keys to watch during the transaction
+        :param value_from_callable: Whether to return the result of transaction or the value
+         returned from :paramref:`func`
         """
-        value_from_callable = kwargs.pop("value_from_callable", False)
-        watch_delay = kwargs.pop("watch_delay", None)
         async with await self.pipeline(True) as pipe:
             while True:
                 try:
