@@ -416,6 +416,9 @@ class RedisCluster(
         self._decodecontext: contextvars.ContextVar[
             Optional[bool],
         ] = contextvars.ContextVar("decode", default=None)
+        self._encodingcontext: contextvars.ContextVar[
+            Optional[str],
+        ] = contextvars.ContextVar("decode", default=None)
 
     @classmethod
     @overload
@@ -721,6 +724,7 @@ class RedisCluster(
                     *args,
                     noreply=self.noreply,
                     decode=kwargs.get("decode", self._decodecontext.get()),
+                    encoding=self._encodingcontext.get(),
                 )
                 if quick_release and not self.requires_wait:
                     released = True
@@ -829,6 +833,7 @@ class RedisCluster(
                             *args,
                             noreply=self.noreply,
                             decode=options.get("decode", self._decodecontext.get()),
+                            encoding=self._encodingcontext.get(),
                         )
                         try:
                             reply = await request
@@ -856,6 +861,7 @@ class RedisCluster(
                         *args,
                         noreply=self.noreply,
                         decode=options.get("decode", self._decodecontext.get()),
+                        encoding=self._encodingcontext.get(),
                         raise_exceptions=False,
                     )
                     reply = await request
@@ -881,6 +887,7 @@ class RedisCluster(
                         *args,
                         noreply=self.noreply,
                         decode=options.get("decode", self._decodecontext.get()),
+                        encoding=self._encodingcontext.get(),
                         raise_exceptions=False,
                     )
                 except ConnectionError as err:
@@ -904,19 +911,31 @@ class RedisCluster(
         return self._merge_result(command, res, **options)
 
     @overload
-    def decoding(self, mode: Literal[False]) -> ContextManager[RedisCluster[bytes]]:
+    def decoding(
+        self, mode: Literal[False], encoding: Optional[str] = None
+    ) -> ContextManager[RedisCluster[bytes]]:
         ...
 
     @overload
-    def decoding(self, mode: Literal[True]) -> ContextManager[RedisCluster[str]]:
+    def decoding(
+        self, mode: Literal[True], encoding: Optional[str] = None
+    ) -> ContextManager[RedisCluster[str]]:
         ...
 
     @contextlib.contextmanager
-    def decoding(self, mode: bool) -> Iterator[RedisCluster[Any]]:
+    @versionadded(version="4.8.0")
+    def decoding(
+        self, mode: bool, encoding: Optional[str] = None
+    ) -> Iterator[RedisCluster[Any]]:
         """
         Context manager to temporarily change the decoding behavior
         of the client
 
+        :param mode: Whether to decode or not
+        :param encoding: Optional encoding to use if decoding. If not provided
+         the :paramref:`~coredis.RedisCluster.encoding` parameter provided to the client will
+         be used.
+
         Example::
 
             client = coredis.RedisCluster(decode_responses=True)
@@ -928,12 +947,15 @@ class RedisCluster(
                     assert await client.get("fubar") == "baz"
 
         """
-        prev = self._decodecontext.get()
+        prev_decode = self._decodecontext.get()
+        prev_encoding = self._encodingcontext.get()
         self._decodecontext.set(mode)
+        self._encodingcontext.set(encoding)
         try:
             yield self
         finally:
-            self._decodecontext.set(prev)
+            self._decodecontext.set(prev_decode)
+            self._encodingcontext.set(prev_encoding)
 
     def pubsub(
         self, ignore_subscribe_messages: bool = False, **kwargs: Any
@@ -988,15 +1010,21 @@ class RedisCluster(
         watches: Optional[Parameters[StringT]] = None,
     ) -> "coredis.pipeline.ClusterPipeline[AnyStr]":
         """
-        Pipelines in cluster mode only provide a subset of the functionality
-        of pipelines in standalone mode.
+        Returns a new pipeline object that can queue multiple commands for
+        batch execution. Pipelines in cluster mode only provide a subset of the
+        functionality of pipelines in standalone mode.
 
         Specifically:
 
-        - Transactions are only supported if all commands in the pipeline
-          only access keys which route to the same node.
         - Each command in the pipeline should only access keys on the same node
-        - Transactions with :paramref:`watch` are not supported.
+        - Transactions are disabled by default and are only supported if all
+          watched keys route to the same node as where the commands in the multi/exec
+          part of the pipeline.
+
+        :param transaction: indicates whether all commands should be executed atomically.
+        :param watches: If :paramref:`transaction` is True these keys are watched for external
+         changes during the transaction.
+
         """
         await self.connection_pool.initialize()
 
@@ -1016,23 +1044,38 @@ class RedisCluster(
             Coroutine[Any, Any, Any],
         ],
         *watches: StringT,
+        value_from_callable: bool = False,
+        watch_delay: Optional[float] = None,
         **kwargs: Any,
     ) -> Any:
         """
         Convenience method for executing the callable :paramref:`func` as a
         transaction while watching all keys specified in :paramref:`watches`.
-        The :paramref:`func` callable should expect a single argument which is a
-        :class:`~coredis.pipeline.ClusterPipeline` instance retrieved
-        by calling :meth:`~coredis.RedisCluster.pipeline`
+
+        :param func: callable should expect a single argument which is a
+         :class:`coredis.pipeline.ClusterPipeline` object retrieved by calling
+         :meth:`~coredis.RedisCluster.pipeline`.
+        :param watches: The keys to watch during the transaction. The keys should route
+         to the same node as the keys touched by the commands in :paramref:`func`
+        :param value_from_callable: Whether to return the result of transaction or the value
+         returned from :paramref:`func`
 
         .. warning:: Cluster transactions can only be run with commands that
-           route to the same node.
+           route to the same slot.
+
+        .. versionchanged:: 4.9.0
+
+           When the transaction is started with :paramref:`watches` the
+           :class:`~coredis.pipeline.ClusterPipeline` instance passed to :paramref:`func`
+           will not start queuing commands until a call to
+           :meth:`~coredis.pipeline.ClusterPipeline.multi` is made. This makes the cluster
+           implementation consistent with :meth:`coredis.Redis.transaction`
         """
-        value_from_callable = kwargs.pop("value_from_callable", False)
-        watch_delay = kwargs.pop("watch_delay", None)
-        async with await self.pipeline(True, watches=watches) as pipe:
+        async with await self.pipeline(True) as pipe:
             while True:
                 try:
+                    if watches:
+                        await pipe.watch(*watches)
                     func_value = await func(pipe)
                     exec_value = await pipe.execute()
                     return func_value if value_from_callable else exec_value
