@@ -49,7 +49,7 @@ class Library(Generic[AnyStr]):
 
     def __init__(
         self,
-        client: coredis.client.AbstractRedis[AnyStr],
+        client: coredis.client.Client[AnyStr],
         name: Optional[StringT] = None,
         code: Optional[StringT] = None,
         replace: bool = False,
@@ -80,7 +80,7 @@ class Library(Generic[AnyStr]):
          redis instance.
         """
         self._client: weakref.ReferenceType[
-            coredis.client.AbstractRedis[AnyStr]
+            coredis.client.Client[AnyStr]
         ] = weakref.ref(client)
         self.name = nativestr(name or self.NAME)
         self.code = (code or self.CODE or "").lstrip()
@@ -92,7 +92,7 @@ class Library(Generic[AnyStr]):
             )
 
     @property
-    def client(self) -> coredis.client.AbstractRedis[AnyStr]:
+    def client(self) -> coredis.client.Client[AnyStr]:
         c = self._client()
         assert c
         return c
@@ -116,9 +116,10 @@ class Library(Generic[AnyStr]):
 
     async def initialize(self: LibraryT, replace: bool = False) -> LibraryT:
         self._functions.clear()
-        if self.code:
-            await self.client.function_load(self.code, replace=replace or self.replace)
         library = (await self.client.function_list(self.name)).get(self.name)
+        if (not library and self.code) or (replace or self.replace):
+            await self.client.function_load(self.code, replace=replace or self.replace)
+            library = (await self.client.function_list(self.name)).get(self.name)
 
         if not library:
             raise FunctionError(f"No library found for {self.name}")
@@ -265,7 +266,7 @@ class Library(Generic[AnyStr]):
         :param runtime_checks: Whether to enable runtime type checking of input arguments
          and return values. (requires :pypi:`beartype`). If :data:`False` the function will
          still get runtime type checking if the environment configuration ``COREDIS_RUNTIME_CHECKS``
-         is set - for details see :ref:`api_reference:runtime type checking`.
+         is set - for details see :ref:`handbook/typing:runtime type checking`.
         :param readonly: If ``True`` forces this function to use :meth:`coredis.Redis.fcall_ro`
 
         :return: A function that has a signature mirroring the decorated function.
@@ -302,8 +303,10 @@ class Library(Generic[AnyStr]):
                 bound_arguments = sig.bind(*a, **k)
                 bound_arguments.apply_defaults()
                 arguments: Dict[str, Any] = bound_arguments.arguments
-                instance: Library[AnyStr]
-                if not isinstance(instance := arguments.pop(first_arg), Library):
+                instance: Library[AnyStr] = arguments.pop(first_arg)
+                if not isinstance(
+                    instance, Library
+                ):  # pyright: reportUnnecessaryIsInstance=false
                     raise RuntimeError(
                         f"{instance.__class__.__name__} is not a subclass of"
                         " coredis.commands.function.Library therefore it's methods cannot be bound "
@@ -344,7 +347,7 @@ class Library(Generic[AnyStr]):
 class Function(Generic[AnyStr]):
     def __init__(
         self,
-        client: coredis.client.AbstractRedis[AnyStr],
+        client: coredis.client.Client[AnyStr],
         library_name: StringT,
         name: StringT,
         readonly: bool = False,
@@ -363,14 +366,14 @@ class Function(Generic[AnyStr]):
             response = await func(keys=["a"], args=[1])
         """
         self._client: weakref.ReferenceType[
-            coredis.client.AbstractRedis[AnyStr]
+            coredis.client.Client[AnyStr]
         ] = weakref.ref(client)
         self.library: Library[AnyStr] = Library[AnyStr](client, library_name)
         self.name = name
         self.readonly = readonly
 
     @property
-    def client(self) -> coredis.client.AbstractRedis[AnyStr]:
+    def client(self) -> coredis.client.Client[AnyStr]:
         c = self._client()
         assert c
         return c
@@ -387,7 +390,7 @@ class Function(Generic[AnyStr]):
         keys: Optional[Parameters[KeyT]] = None,
         args: Optional[Parameters[ValueT]] = None,
         *,
-        client: Optional[coredis.client.AbstractRedis[AnyStr]] = None,
+        client: Optional[coredis.client.Client[AnyStr]] = None,
         readonly: Optional[bool] = None,
     ) -> ResponseType:
         """
