@@ -1,6 +1,7 @@
 from __future__ import annotations
 
 import asyncio
+import inspect
 import threading
 from asyncio import CancelledError
 from concurrent.futures import Future
@@ -9,7 +10,7 @@ from typing import TYPE_CHECKING, Any, cast
 
 from deprecated.sphinx import versionadded
 
-from coredis._utils import b, hash_slot, nativestr
+from coredis._utils import CaseAndEncodingInsensitiveEnum, b, hash_slot, nativestr
 from coredis.commands.constants import CommandName
 from coredis.connection import BaseConnection, Connection
 from coredis.exceptions import ConnectionError, PubSubError, TimeoutError
@@ -41,21 +42,43 @@ T = TypeVar("T")
 
 PoolT = TypeVar("PoolT", bound="coredis.pool.ConnectionPool")
 
+#: Callables for message handler callbacks. The callbacks
+#:  can be sync or async.
+SubscriptionCallback = Union[
+    Callable[[PubSubMessage], Awaitable[None]], Callable[[PubSubMessage], None]
+]
+
+
+class PubSubMessageTypes(CaseAndEncodingInsensitiveEnum):
+    MESSAGE = b"message"
+    PMESSAGE = b"pmessage"
+    SMESSAGE = b"smessage"
+    SUBSCRIBE = b"subscribe"
+    UNSUBSCRIBE = b"unsubscribe"
+    PSUBSCRIBE = b"psubscribe"
+    PUNSUBSCRIBE = b"punsubscribe"
+    SSUBSCRIBE = b"ssubscribe"
+    SUNSUBSCRIBE = b"sunsubscribe"
+
 
 class BasePubSub(Generic[AnyStr, PoolT]):
-    PUBLISH_MESSAGE_TYPES = {b"message", b"pmessage"}
+    PUBLISH_MESSAGE_TYPES = {
+        PubSubMessageTypes.MESSAGE.value,
+        PubSubMessageTypes.PMESSAGE.value,
+    }
     SUBUNSUB_MESSAGE_TYPES = {
-        b"subscribe",
-        b"psubscribe",
-        b"unsubscribe",
-        b"punsubscribe",
+        PubSubMessageTypes.SUBSCRIBE.value,
+        PubSubMessageTypes.PSUBSCRIBE.value,
+        PubSubMessageTypes.UNSUBSCRIBE.value,
+        PubSubMessageTypes.PUNSUBSCRIBE.value,
     }
     UNSUBSCRIBE_MESSAGE_TYPES = {
-        b"unsubscribe",
-        b"punsubscribe",
+        PubSubMessageTypes.UNSUBSCRIBE.value,
+        PubSubMessageTypes.PUNSUBSCRIBE.value,
     }
-    channels: MutableMapping[StringT, Optional[Callable[[PubSubMessage], None]]]
-    patterns: MutableMapping[StringT, Optional[Callable[[PubSubMessage], None]]]
+
+    channels: MutableMapping[StringT, Optional[SubscriptionCallback]]
+    patterns: MutableMapping[StringT, Optional[SubscriptionCallback]]
 
     def __init__(
         self,
@@ -221,20 +244,18 @@ class BasePubSub(Generic[AnyStr, PoolT]):
     async def psubscribe(
         self,
         *patterns: StringT,
-        **pattern_handlers: Optional[Callable[[PubSubMessage], None]],
+        **pattern_handlers: Optional[SubscriptionCallback],
     ) -> None:
         """
         Subscribes to channel patterns. Patterns supplied as keyword arguments
         expect a pattern name as the key and a callable as the value. A
         pattern's callable will be invoked automatically when a message is
         received on that pattern rather than producing a message via
-        ``listen()``.
+        :meth:`listen`.
         """
         await self._ensure_encoding()
 
-        new_patterns: MutableMapping[
-            StringT, Optional[Callable[[PubSubMessage], None]]
-        ] = {}
+        new_patterns: MutableMapping[StringT, Optional[SubscriptionCallback]] = {}
         new_patterns.update(dict.fromkeys(map(self.encode, patterns)))
 
         for pattern, handler in pattern_handlers.items():
@@ -256,21 +277,19 @@ class BasePubSub(Generic[AnyStr, PoolT]):
     async def subscribe(
         self,
         *channels: StringT,
-        **channel_handlers: Optional[Callable[[PubSubMessage], None]],
+        **channel_handlers: Optional[SubscriptionCallback],
     ) -> None:
         """
         Subscribes to channels. Channels supplied as keyword arguments expect
         a channel name as the key and a callable as the value. A channel's
         callable will be invoked automatically when a message is received on
-        that channel rather than producing a message via ``listen()`` or
-        ``get_message()``.
+        that channel rather than producing a message via :meth:`listen` or
+        :meth:`get_message`.
         """
 
         await self._ensure_encoding()
 
-        new_channels: MutableMapping[
-            StringT, Optional[Callable[[PubSubMessage], None]]
-        ] = {}
+        new_channels: MutableMapping[StringT, Optional[SubscriptionCallback]] = {}
         new_channels.update(dict.fromkeys(map(self.encode, channels)))
 
         for channel, handler in channel_handlers.items():
@@ -298,7 +317,7 @@ class BasePubSub(Generic[AnyStr, PoolT]):
         if self.subscribed:
             response = await self.parse_response(block=True)
             if response:
-                return self.handle_message(response)
+                return await self.handle_message(response)
         return None
 
     async def get_message(
@@ -317,11 +336,11 @@ class BasePubSub(Generic[AnyStr, PoolT]):
         response = await self.parse_response(block=False, timeout=timeout)
 
         if response:
-            return self.handle_message(response, ignore_subscribe_messages)
+            return await self.handle_message(response, ignore_subscribe_messages)
 
         return None
 
-    def handle_message(
+    async def handle_message(
         self, response: ResponseType, ignore_subscribe_messages: bool = False
     ) -> Optional[PubSubMessage]:
         """
@@ -347,7 +366,7 @@ class BasePubSub(Generic[AnyStr, PoolT]):
                 data=cast(int, r[2]),
             )
         elif message_type in self.PUBLISH_MESSAGE_TYPES:
-            if message_type == b"pmessage":
+            if message_type == PubSubMessageTypes.PMESSAGE:
                 message = PubSubMessage(
                     type="pmessage",
                     pattern=cast(StringT, r[1]),
@@ -366,7 +385,7 @@ class BasePubSub(Generic[AnyStr, PoolT]):
 
         # if this is an unsubscribe message, remove it from memory
         if message_type in self.UNSUBSCRIBE_MESSAGE_TYPES:
-            if message_type == b"punsubscribe":
+            if message_type == PubSubMessageTypes.PUNSUBSCRIBE:
                 subscribed_dict = self.patterns
             else:
                 subscribed_dict = self.channels
@@ -374,13 +393,15 @@ class BasePubSub(Generic[AnyStr, PoolT]):
 
         if message_type in self.PUBLISH_MESSAGE_TYPES:
             handler = None
-            if message_type == b"pmessage" and message["pattern"]:
+            if message_type == PubSubMessageTypes.PMESSAGE and message["pattern"]:
                 handler = self.patterns.get(message["pattern"], None)
             elif message["channel"]:
                 handler = self.channels.get(message["channel"], None)
 
             if handler:
-                handler(message)
+                handler_response = handler(message)
+                if inspect.isawaitable(handler_response):
+                    await handler_response
                 return None
         else:
             # this is a subscribe/unsubscribe message. ignore if we don't
@@ -396,8 +417,8 @@ class BasePubSub(Generic[AnyStr, PoolT]):
         Run the listeners in a thread. For each message received on a
         subscribed channel or pattern the registered handlers will be invoked.
 
-        To stop listening invoke :meth:`PubSubWorkerThread.stop` on the returned
-        instead of :class:`PubSubWorkerThread`.
+        To stop listening invoke :meth:`~coredis.commands.pubsub.PubSubWorkerThread.stop`
+        on the returned instance
         """
         for channel, handler in self.channels.items():
             if handler is None:
@@ -473,9 +494,15 @@ class ShardedPubSub(BasePubSub[AnyStr, "coredis.pool.ClusterConnectionPool"]):
        **NOT** support pattern based subscriptions.
     """
 
-    PUBLISH_MESSAGE_TYPES = {b"message", b"smessage"}
-    SUBUNSUB_MESSAGE_TYPES = {b"ssubscribe", b"sunsubscribe"}
-    UNSUBSCRIBE_MESSAGE_TYPES = {b"sunsubscribe"}
+    PUBLISH_MESSAGE_TYPES = {
+        PubSubMessageTypes.MESSAGE.value,
+        PubSubMessageTypes.SMESSAGE.value,
+    }
+    SUBUNSUB_MESSAGE_TYPES = {
+        PubSubMessageTypes.SSUBSCRIBE.value,
+        PubSubMessageTypes.SUNSUBSCRIBE.value,
+    }
+    UNSUBSCRIBE_MESSAGE_TYPES = {PubSubMessageTypes.SUNSUBSCRIBE.value}
 
     def __init__(
         self,
@@ -492,21 +519,19 @@ class ShardedPubSub(BasePubSub[AnyStr, "coredis.pool.ClusterConnectionPool"]):
     async def subscribe(
         self,
         *channels: StringT,
-        **channel_handlers: Optional[Callable[[PubSubMessage], None]],
+        **channel_handlers: Optional[SubscriptionCallback],
     ) -> None:
         """
         :param channels: The shard channels to subscribe to.
         :param channel_handlers: Channels supplied as keyword arguments expect
          a channel name as the key and a callable as the value. A channel's
          callable will be invoked automatically when a message is received on
-         that channel rather than producing a message via ``listen()`` or
-         ``get_message()``.
+         that channel rather than producing a message via :meth:`listen` or
+         :meth:`get_message`.
         """
 
         await self._ensure_encoding()
-        new_channels: MutableMapping[
-            StringT, Optional[Callable[[PubSubMessage], None]]
-        ] = {}
+        new_channels: MutableMapping[StringT, Optional[SubscriptionCallback]] = {}
         new_channels.update(dict.fromkeys(map(self.encode, channels)))
 
         for channel, handler in channel_handlers.items():
@@ -531,7 +556,7 @@ class ShardedPubSub(BasePubSub[AnyStr, "coredis.pool.ClusterConnectionPool"]):
     async def psubscribe(
         self,
         *patterns: StringT,
-        **pattern_handlers: Optional[Callable[[PubSubMessage], None]],
+        **pattern_handlers: Optional[SubscriptionCallback],
     ) -> None:
         """
         Not available in sharded pubsub
@@ -699,7 +724,6 @@ class PubSubWorkerThread(threading.Thread):
     def stop(self) -> None:
         """
         Stop the worker thread from processing any more messages
-        :return:
         """
         if self._future:
             self._future.cancel()
