@@ -3,6 +3,7 @@ from __future__ import annotations
 import asyncio
 import dataclasses
 import inspect
+import itertools
 import os
 import socket
 import ssl
@@ -35,6 +36,7 @@ from coredis.typing import (
     Optional,
     ResponseType,
     Set,
+    Tuple,
     TypeVar,
     Union,
     ValueT,
@@ -49,7 +51,8 @@ if TYPE_CHECKING:
 @dataclasses.dataclass
 class Request:
     command: bytes
-    decode: Optional[bool] = None
+    decode: bool
+    encoding: Optional[str] = None
     raise_exceptions: bool = True
     future: asyncio.Future[ResponseType] = dataclasses.field(
         default_factory=lambda: asyncio.get_running_loop().create_future()
@@ -61,6 +64,14 @@ class Request:
             self.future.set_exception(TimeoutError())
 
 
+@dataclasses.dataclass
+class CommandInvocation:
+    command: bytes
+    args: Tuple[ValueT, ...]
+    decode: Optional[bool]
+    encoding: Optional[str]
+
+
 class RedisSSLContext:
     context: Optional[ssl.SSLContext]
 
@@ -171,7 +182,7 @@ class BaseConnection(asyncio.BaseProtocol):
         self.last_request_processed_at: Optional[float] = None
 
         self._transport: Optional[asyncio.Transport] = None
-        self._parser = Parser(encoding, decode_responses)
+        self._parser = Parser()
         self._read_flag = asyncio.Event()
         self.packer = Packer(self.encoding)
         self.push_messages: asyncio.Queue[ResponseType] = asyncio.Queue()
@@ -321,7 +332,7 @@ class BaseConnection(asyncio.BaseProtocol):
             return
 
         request = self._requests.popleft()
-        response = self._parser.get_response(request.decode)
+        response = self._parser.get_response(request.decode, request.encoding)
         while not isinstance(
             response,
             NotEnoughData,
@@ -345,7 +356,7 @@ class BaseConnection(asyncio.BaseProtocol):
             except IndexError:
                 return
 
-            response = self._parser.get_response(request.decode)
+            response = self._parser.get_response(request.decode, request.encoding)
 
         # In case the first request pulled from the queue doesn't have enough data
         # to process, put it back to the start of the queue for the next iteration
@@ -486,7 +497,9 @@ class BaseConnection(asyncio.BaseProtocol):
             )
 
         message = self._parser.get_response(
-            bool(decode) if decode is not None else None, push_message_types
+            bool(decode) if decode is not None else self.decode_responses,
+            self.encoding,
+            push_message_types,
         )
         while isinstance(
             message,
@@ -499,7 +512,9 @@ class BaseConnection(asyncio.BaseProtocol):
             except asyncio.TimeoutError:
                 raise TimeoutError
             message = self._parser.get_response(
-                bool(decode) if decode is not None else None, push_message_types
+                bool(decode) if decode is not None else self.decode_responses,
+                self.encoding,
+                push_message_types,
             )
         return message
 
@@ -533,6 +548,7 @@ class BaseConnection(asyncio.BaseProtocol):
         *args: ValueT,
         noreply: Optional[bool] = None,
         decode: Optional[ValueT] = None,
+        encoding: Optional[str] = None,
         raise_exceptions: bool = True,
     ) -> asyncio.Future[ResponseType]:
         """
@@ -556,7 +572,8 @@ class BaseConnection(asyncio.BaseProtocol):
         if not (self.noreply_set or noreply):
             request = Request(
                 command,
-                bool(decode) if decode is not None else None,
+                bool(decode) if decode is not None else self.decode_responses,
+                encoding or self.encoding,
                 raise_exceptions,
             )
             self._requests.append(request)
@@ -570,6 +587,41 @@ class BaseConnection(asyncio.BaseProtocol):
             none.set_result(None)
             return none
 
+    async def create_requests(
+        self,
+        commands: List[CommandInvocation],
+        raise_exceptions: bool = True,
+    ) -> List[asyncio.Future[ResponseType]]:
+        """
+        Send multiple commands to the redis server
+        """
+
+        if not self.is_connected:
+            await self.connect()
+
+        self._send_packed_command(
+            self.packer.pack_commands(
+                list(itertools.chain((cmd.command,) + cmd.args for cmd in commands))
+            )
+        )
+
+        self.last_active_at = time.time()
+        requests: List[asyncio.Future[ResponseType]] = []
+        for cmd in commands:
+            request = Request(
+                cmd.command,
+                bool(cmd.decode) if cmd.decode is not None else self.decode_responses,
+                cmd.encoding or self.encoding,
+                raise_exceptions,
+            )
+            self._requests.append(request)
+            if self._stream_timeout is not None:
+                asyncio.get_running_loop().call_later(
+                    self._stream_timeout, request.enforce_deadline
+                )
+            requests.append(request.future)
+        return requests
+
     def disconnect(self) -> None:
         """
         Disconnect from the Redis server
