@@ -1,25 +1,21 @@
 from __future__ import annotations
 
-import os
 import random
-from asyncio import AbstractEventLoop
+import ssl
+import weakref
 from typing import Any, cast, overload
 
-from deprecated.sphinx import deprecated, versionadded
-
 from coredis import Redis
 from coredis._utils import nativestr
-from coredis.commands.constants import CommandName
-from coredis.connection import Connection, RedisSSLContext
+from coredis.cache import AbstractCache
+from coredis.connection import Connection
 from coredis.exceptions import (
     ConnectionError,
     PrimaryNotFoundError,
-    ReadOnlyError,
     ReplicaNotFoundError,
     ResponseError,
     TimeoutError,
 )
-from coredis.parsers import BaseParser, DefaultParser
 from coredis.pool import ConnectionPool
 from coredis.typing import (
     AnyStr,
@@ -29,8 +25,6 @@ from coredis.typing import (
     List,
     Literal,
     Optional,
-    ResponseType,
-    Set,
     StringT,
     Tuple,
     Type,
@@ -50,19 +44,16 @@ class SentinelManagedConnection(Connection, Generic[AnyStr]):
         retry_on_timeout: bool = False,
         stream_timeout: Optional[float] = None,
         connect_timeout: Optional[float] = None,
-        ssl_context: Optional[RedisSSLContext] = None,
-        parser_class: Type[BaseParser] = DefaultParser,
-        reader_read_size: int = 65535,
+        ssl_context: Optional[ssl.SSLContext] = None,
         encoding: str = "utf-8",
         decode_responses: bool = False,
         socket_keepalive: Optional[bool] = None,
         socket_keepalive_options: Optional[Dict[int, Union[int, bytes]]] = None,
         *,
         client_name: Optional[str] = None,
-        loop: Optional[AbstractEventLoop] = None,
         protocol_version: Literal[2, 3] = 2,
     ):
-        self.connection_pool: SentinelConnectionPool = connection_pool
+        self.connection_pool: SentinelConnectionPool = weakref.proxy(connection_pool)
         super().__init__(
             host=host,
             port=port,
@@ -73,14 +64,11 @@ class SentinelManagedConnection(Connection, Generic[AnyStr]):
             stream_timeout=stream_timeout,
             connect_timeout=connect_timeout,
             ssl_context=ssl_context,
-            parser_class=parser_class,
-            reader_read_size=reader_read_size,
             encoding=encoding,
             decode_responses=decode_responses,
             socket_keepalive=socket_keepalive,
             socket_keepalive_options=socket_keepalive_options,
             client_name=client_name,
-            loop=loop,
             protocol_version=protocol_version,
         )
 
@@ -96,52 +84,24 @@ class SentinelManagedConnection(Connection, Generic[AnyStr]):
     async def connect_to(self, address: Tuple[str, int]) -> None:
         self.host, self.port = address
         await super().connect()
-        if self.connection_pool.check_connection:
-            await self.send_command(CommandName.PING)
-            if await self.read_response(decode=False) != b"PONG":
-                raise ConnectionError("PING failed")
 
     async def connect(self) -> None:
-        if self._reader and self._writer:
-            return  # already connected
-        if self.connection_pool.is_primary:
-            await self.connect_to(await self.connection_pool.get_primary_address())
-        else:
-            for slave in await self.connection_pool.rotate_replicas():
-                try:
-                    return await self.connect_to(slave)
-                except ConnectionError:
-                    continue
-            raise ReplicaNotFoundError  # Never be here
-
-    async def read_response(
-        self,
-        decode: Optional[bool] = None,
-        push_message_types: Optional[Set[bytes]] = None,
-    ) -> ResponseType:
-        try:
-            return await super().read_response(
-                decode=decode, push_message_types=push_message_types
-            )
-        except ReadOnlyError:
+        if not self.is_connected:
             if self.connection_pool.is_primary:
-                # When talking to a master, a ReadOnlyError when likely
-                # indicates that the previous master that we're still connected
-                # to has been demoted to a slave and there's a new master.
-                # calling disconnect will force the connection to re-query
-                # sentinel during the next connect() attempt.
-                self.disconnect()
-                raise ConnectionError("The previous master is now a replica")
-            raise
+                await self.connect_to(await self.connection_pool.get_primary_address())
+            else:
+                for replica in await self.connection_pool.rotate_replicas():
+                    try:
+                        return await self.connect_to(replica)
+                    except ConnectionError:
+                        continue
+                raise ReplicaNotFoundError  # Never be here
+        return None
 
 
 class SentinelConnectionPool(ConnectionPool):
     """
     Sentinel backed connection pool.
-
-    If :paramref:`check_connection` is ``True``,
-    :class:`~coredis.sentinel.SentinelManagedConnection`
-    sends a ``PING`` command right after establishing the connection.
     """
 
     primary_address: Optional[Tuple[str, int]]
@@ -156,7 +116,6 @@ class SentinelConnectionPool(ConnectionPool):
         **kwargs: Any,
     ):
         self.is_primary = is_primary
-        self.check_connection = check_connection
         kwargs["connection_class"] = cast(
             Type[Connection],
             kwargs.get("connection_class", SentinelManagedConnection[AnyStr]),
@@ -165,12 +124,13 @@ class SentinelConnectionPool(ConnectionPool):
         self.connection_kwargs["connection_pool"] = self
         self.service_name = nativestr(service_name)
         self.sentinel_manager = sentinel_manager
+        self.check_connection = check_connection
 
     def __repr__(self) -> str:
-        return "{}<service={}({})".format(
-            type(self).__name__,
-            self.service_name,
-            self.is_primary and "master" or "slave",
+        return (
+            f"{type(self).__name__}"
+            f"<service={self.service_name}"
+            f"({'primary' if self.is_primary else 'replica'})"
         )
 
     def reset(self) -> None:
@@ -186,7 +146,7 @@ class SentinelConnectionPool(ConnectionPool):
             if self.primary_address is None:
                 self.primary_address = primary_address
             elif primary_address != self.primary_address:
-                # Master address changed, disconnect all clients in this pool
+                # Primary address changed, disconnect all clients in this pool
                 self.disconnect()
         return primary_address
 
@@ -208,25 +168,9 @@ class SentinelConnectionPool(ConnectionPool):
             pass
         raise ReplicaNotFoundError("No replica found for %r" % (self.service_name))
 
-    def checkpid(self) -> None:
-        if self.pid != os.getpid():
-            self.disconnect()
-            self.reset()
-            SentinelConnectionPool(
-                self.service_name,
-                self.sentinel_manager,
-                is_primary=self.is_primary,
-                check_connection=self.check_connection,
-                connection_class=self.connection_class,
-                max_connections=self.max_connections,
-                **self.connection_kwargs,
-            )
-
 
 class Sentinel(Generic[AnyStr]):
     """
-    Redis Sentinel client
-
     Example use::
 
         from coredis.sentinel import Sentinel
@@ -246,6 +190,7 @@ class Sentinel(Generic[AnyStr]):
         min_other_sentinels: int = ...,
         sentinel_kwargs: Optional[Dict[str, Any]] = ...,
         decode_responses: Literal[False] = ...,
+        cache: Optional[AbstractCache] = None,
         **connection_kwargs: Any,
     ):
         ...
@@ -257,6 +202,7 @@ class Sentinel(Generic[AnyStr]):
         min_other_sentinels: int = ...,
         sentinel_kwargs: Optional[Dict[str, Any]] = ...,
         decode_responses: Literal[True] = ...,
+        cache: Optional[AbstractCache] = None,
         **connection_kwargs: Any,
     ):
         ...
@@ -267,9 +213,15 @@ class Sentinel(Generic[AnyStr]):
         min_other_sentinels: int = 0,
         sentinel_kwargs: Optional[Dict[str, Any]] = None,
         decode_responses: bool = False,
+        cache: Optional[AbstractCache] = None,
         **connection_kwargs: Any,
     ):
         """
+        Changes
+          - .. versionadded:: 3.10.0
+               Accept :paramref:`cache` parameter to be used with primaries
+               and replicas returned from the sentinel instance.
+
         :param sentinels: is a list of sentinel nodes. Each node is represented by
          a pair (hostname, port).
         :param min_other_sentinels: defined a minimum number of peers for a sentinel.
@@ -280,14 +232,15 @@ class Sentinel(Generic[AnyStr]):
          a normal Redis connection can be specified here. If :paramref:`sentinel_kwargs` is
          not specified, ``stream_timeout``, ``socket_keepalive``, ``decode_responses``
          and ``protocol_version`` options specified in :paramref:`connection_kwargs` will be used.
-        :param decode_responses:
+        :param cache: If provided the cache will be shared between both primaries and replicas
+         returned by this sentinel.
         :param connection_kwargs: are keyword arguments that will be used when
          establishing a connection to a Redis server (i.e. are passed on to the
          constructor of :class:`Redis` for all primary and replicas).
         """
         # if sentinel_kwargs isn't defined, use the socket_* options from
         # connection_kwargs
-        if sentinel_kwargs is None:
+        if not sentinel_kwargs:
             sentinel_kwargs = {
                 k: v
                 for k, v in iter(connection_kwargs.items())
@@ -303,7 +256,7 @@ class Sentinel(Generic[AnyStr]):
         self.sentinel_kwargs = sentinel_kwargs
         self.min_other_sentinels = min_other_sentinels
         self.connection_kwargs = connection_kwargs
-
+        self.__cache = cache
         self.connection_kwargs["decode_responses"] = self.sentinel_kwargs[
             "decode_responses"
         ] = decode_responses
@@ -347,14 +300,13 @@ class Sentinel(Generic[AnyStr]):
             replicas_alive.append((nativestr(replica["ip"]), int(replica["port"])))
         return replicas_alive
 
-    @versionadded("Replaces deprecated :meth:`discover_master`", version="3.1.0")
     async def discover_primary(self, service_name: str) -> Tuple[str, int]:
         """
         Asks sentinel servers for the Redis primary's address corresponding
         to the service labeled :paramref:`service_name`.
 
-        :return: A pair (address, port) or raises MasterNotFoundError if no
-         primary is found.
+        :return: A pair (address, port) or raises :exc:`~coredis.exceptions.PrimaryNotFoundError`
+         if no primary is found.
         """
         for sentinel_no, sentinel in enumerate(self.sentinels):
             try:
@@ -371,9 +323,8 @@ class Sentinel(Generic[AnyStr]):
                 return nativestr(state["ip"]), int(state["port"])
         raise PrimaryNotFoundError(f"No primary found for {service_name!r}")
 
-    @versionadded("Replaces deprecated :meth:`discover_slaves`", version="3.1.0")
     async def discover_replicas(self, service_name: str) -> List[Tuple[str, int]]:
-        """Returns a list of alive slaves for service :paramref:`service_name`"""
+        """Returns a list of alive replicas for service :paramref:`service_name`"""
         for sentinel in self.sentinels:
             try:
                 replicas = await sentinel.sentinel_replicas(service_name)
@@ -406,7 +357,6 @@ class Sentinel(Generic[AnyStr]):
     ) -> Redis[str]:
         ...
 
-    @versionadded("Replaces deprecated :meth:`master_for`", version="3.1.0")
     def primary_for(
         self,
         service_name: str,
@@ -416,7 +366,7 @@ class Sentinel(Generic[AnyStr]):
         **kwargs: Any,
     ) -> Union[Redis[bytes], Redis[str]]:
         """
-        Returns a redis client instance for the :paramref:`service_name` master.
+        Returns a redis client instance for the :paramref:`service_name` primary.
 
         A :class:`coredis.sentinel.SentinelConnectionPool` class is used to
         retrive the primary's address before establishing a new connection.
@@ -443,7 +393,8 @@ class Sentinel(Generic[AnyStr]):
                 service_name,
                 self,
                 **connection_kwargs,
-            )
+            ),
+            cache=self.__cache,
         )
 
     @overload
@@ -466,7 +417,6 @@ class Sentinel(Generic[AnyStr]):
     ) -> Redis[str]:
         ...
 
-    @versionadded("Replaces deprecated :meth:`slave_for`", version="3.1.0")
     def replica_for(
         self,
         service_name: str,
@@ -475,9 +425,9 @@ class Sentinel(Generic[AnyStr]):
         **kwargs: Any,
     ) -> Union[Redis[bytes], Redis[str]]:
         """
-        Returns redis client instance for the :paramref:`service_name` slave(s).
+        Returns redis client instance for the :paramref:`service_name` replica(s).
 
-        A SentinelConnectionPool class is used to retrive the slave's
+        A SentinelConnectionPool class is used to retrieve the replica's
         address before establishing a new connection.
 
         By default clients will be a redis.Redis instance. Specify a
@@ -499,55 +449,6 @@ class Sentinel(Generic[AnyStr]):
                 service_name,
                 self,
                 **connection_kwargs,
-            )
-        )
-
-    @deprecated(version="3.1.0", reason="Use :meth:`discover_primary()` instead")
-    async def discover_master(self, service_name: str) -> Tuple[str, int]:
-        """
-        :meta private:
-        """
-        return await self.discover_primary(service_name)
-
-    @deprecated(version="3.1.0", reason="Use :meth:`discover_replicas()` instead")
-    async def discover_slaves(self, service_name: str) -> List[Tuple[str, int]]:
-        """
-        :meta private:
-        """
-        return await self.discover_replicas(service_name)
-
-    @deprecated(version="3.1.0", reason="Use :meth:`replica_for()` instead")
-    def slave_for(
-        self,
-        service_name: str,
-        redis_class: Type[Redis[Any]] = Redis[Any],
-        connection_pool_class: Type[SentinelConnectionPool] = SentinelConnectionPool,
-        **kwargs: Any,
-    ) -> Redis[Any]:
-        """
-        :meta private:
-        """
-        return self.replica_for(  # pyright: ignore
-            service_name,
-            redis_class=redis_class,
-            connection_pool_class=connection_pool_class,
-            **kwargs,
-        )
-
-    @deprecated(version="3.1.0", reason="Use :meth:`primary_for()` instead")
-    def master_for(
-        self,
-        service_name: str,
-        redis_class: Type[Redis[Any]] = Redis[Any],
-        connection_pool_class: Type[SentinelConnectionPool] = SentinelConnectionPool,
-        **kwargs: Any,
-    ) -> Redis[Any]:
-        """
-        :meta private:
-        """
-        return self.primary_for(  # pyright: ignore
-            service_name,
-            redis_class=redis_class,
-            connection_pool_class=connection_pool_class,
-            **kwargs,
+            ),
+            cache=self.__cache,
         )
