@@ -10,6 +10,7 @@ from typing import (
     Awaitable,
     Callable,
     ClassVar,
+    ContextManager,
     Coroutine,
     Dict,
     Generator,
@@ -200,6 +201,7 @@ __all__ = [
     "Callable",
     "ClassVar",
     "CommandArgList",
+    "ContextManager",
     "Coroutine",
     "Deque",
     "Dict",
